import os

ROOT_DIR = os.path.abspath(os.path.dirname(__file__))
BIP39_DIR = os.path.normpath(os.path.join(ROOT_DIR, 'bip39_word_list'))
SIGHASH_ALL = 0x00000001
SIGHASH_NONE = 0x00000002
SIGHASH_SINGLE = 0x00000003
MAX_HEX = 0xfffffffffffffffffffffffffffffffebaaedce6af48a03bbfd25e8cd0364141
MAX_DEC = 115792089237316195423570985008687907852837564279074904382605163141518161494337
ZERO_BYTE = (b'\x04\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
             b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
             b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00')
PREFIX_0 = b'\x00'
PREFIX_1 = b'\x01'
PREFIX_2 = b'\x02'
PREFIX_3 = b'\x03'
PREFIX_4 = b'\x04'
PREFIX_5 = b'\x05'
PREFIX_6 = b'\x06'
PREFIX_7 = b'\x07'
PREFIX_8 = b'\x08'
PREFIX_9 = b'\x09'
PREFIX_80 = b'\x80'
PREFIX_LIST = [PREFIX_0, PREFIX_1, PREFIX_2, PREFIX_3, PREFIX_4, PREFIX_5, PREFIX_6, PREFIX_7, PREFIX_8, PREFIX_9, PREFIX_80]
MAINNET_SEGWIT_ADDRESS_BYTE_PREFIX = b'\x03\x03\x00\x02\x03'

WIF_UNCOMPRESSED_PREFIX = '5'
WIF_COMPRESSED_PREFIX = 'K'
WIF_COMPRESSED_PREFIX_2 = 'L'

WIF_PREFIX = [WIF_UNCOMPRESSED_PREFIX, WIF_COMPRESSED_PREFIX, WIF_COMPRESSED_PREFIX_2]
BASE58_CHARS = '123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz'
MAINNET_PREFIX = b'\x80'




# Python Secp256k1 Library
import sys, os, ctypes


def load_dynamic_lib(lib_path):
    if not os.path.isfile(lib_path):
        raise ValueError(f"File {lib_path} not found")
    try:
        lib = ctypes.CDLL(lib_path)
    except Exception as e:
        raise ValueError(f"Error loading {lib_path}: {e}")
    return lib


def Load_cPack():
    dirPath = os.path.dirname(os.path.realpath(__file__))
    if 'win' in sys.platform.lower():

        secFile = dirPath + '\\_secp256k2.dll'
    elif sys.platform.lower() in ['linux', 'darwin']:
        secFile = dirPath + '/_secp256k2.so'
    else:
        raise EnvironmentError("Unsupported platform")
    return load_dynamic_lib(secFile)


import os
import sys
import ctypes

import math, pickle
from .constants import (
    MAX_HEX,
    MAX_DEC,
    ZERO_BYTE,
    BASE58_CHARS,
    PREFIX_0,
    PREFIX_80,
    PREFIX_9,
    PREFIX_8,
    PREFIX_7,
    PREFIX_6,
    PREFIX_5,
    PREFIX_4,
    PREFIX_3,
    PREFIX_2,
    PREFIX_1
)
from .Loader import Load_cPack
from . import utils

Fuzz = Load_cPack()

for func_name, args in utils.argtypes_dict.items():
    getattr(Fuzz, func_name).argtypes = args

for func_name, restype in utils.restype_dict.items():
    getattr(Fuzz, func_name).restype = restype

Fuzz.init_secp256_lib()


class Base58k1:

    def __init__(self):
        super().__init__()

    # =============================================================================

    @staticmethod
    def fuzz256k1(self, result):
        return ctypes.cast(result, ctypes.c_char_p).value.decode('utf8')

    def b58py(self, data):
        B58 = "123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz"

        if data[0] == 0:
            return "1" + self.b58py(data[1:])

        x = sum([v * (256 ** i) for i, v in enumerate(data[::-1])])
        ret = ""
        while x > 0:
            ret = B58[x % 58] + ret
            x = x // 58

        return ret

    # ==============================================================================
    def b58_encode(self, inp_bytes):
        res = Fuzz.b58_encode(inp_bytes, len(inp_bytes))
        addr = self.fuzz256k1(res)
        Fuzz.free_memory(res)
        return addr

    # ==============================================================================
    def b58_decode(self, inp):
        res = Fuzz.b58_decode(inp.encode("utf-8"))
        addr = self.fuzz256k1(res)
        Fuzz.free_memory(res)
        return addr


def get_sha256(input_bytes):
    digest_bytes = PREFIX_0 * 32
    if type(input_bytes) == str:
        input_bytes = input_bytes.encode("utf-8")
    #    MiniKey example
    Fuzz.get_sha256(input_bytes, len(input_bytes), digest_bytes)
    return digest_bytes


def fl(s, length=64):
    fixed = None
    if type(s) == int:
        fixed = hex(s)[2:].zfill(length)
    elif type(s) == str:
        fixed = s[2:].zfill(length) if s[:2].lower() == '0x' else s.zfill(length)
    elif type(s) == bytes:
        fixed = PREFIX_0 * 32 - len(s) + s
    else:
        ValueError("[Error] Input format [Integer] [Hex] [Bytes] allowed only. Detected : ", type(s))
    return fixed


class Contactor:
    def __init__(self):
        super(Contactor, self).__init__()
        self.b58k1 = Base58k1()

    # ==============================================================================

    def version(self):
        Fuzz.version()

    # ==============================================================================
    def fuzz256k1(self, result):
        return ctypes.cast(result, ctypes.c_char_p).value.decode('utf8')

    # ==============================================================================
    def Decimal_To_Addr(self, decimal, addr_type: int, compress: bool = True):
        if decimal < 0:
            decimal = MAX_HEX + decimal
        pass_int_value = fl(decimal).encode('utf-8')
        res = Fuzz.privatekey_to_address(addr_type, compress, pass_int_value)
        addr = self.fuzz256k1(res)
        Fuzz.free_memory(res)
        return addr

    # =============================================================================
    def Decimal_To_Wif(self, decimal, compress=True):
        inp = ''
        suff = '01' if compress == True else ''
        if type(decimal) in [int, str]:
            inp = bytes.fromhex('80' + fl(decimal) + suff)
        elif type(decimal) == bytes:
            inp = b'\x80' + fl(decimal) + bytes.fromhex(suff)
        else:
            ValueError("[Error] Input format [Integer] [Hex] [Bytes] allowed only. Detected : ", type(decimal))
        if inp != '':
            res = get_sha256(get_sha256(inp))
            return self.b58k1.b58_encode(inp + res[:4])
        else:
            return inp

    # ==============================================================================
    @staticmethod
    def Decimal_To_RIPEMD160(self, dec, addr_type, compress=True):
        if dec < 0:
            dec = MAX_HEX + dec
        pass_int_value = fl(dec).encode('utf8')
        res = PREFIX_0 * 20
        Fuzz.privatekey_to_h160(addr_type, compress, pass_int_value, res)
        return res

    # ==============================================================================
    def Decimal_To_RIPEMD160_DIGEST(self, dec, addr_type, compress=True):
        res = self.Decimal_To_RIPEMD160(dec, addr_type, compress)
        return self.byter(res)

    # ==============================================================================
    def Decimal_To_ETH_Addr(self, dec):
        if dec < 0: dec = MAX_HEX + dec
        pass_int_value = fl(dec).encode('utf8')
        res = Fuzz.privatekey_to_ETH_address(pass_int_value)
        addr = self.fuzz256k1(res)
        Fuzz.free_memory(res)
        return '0x' + addr

    # ==============================================================================

    @staticmethod
    def Decimal_To_ETH_Addr_Bytes(self, dec):
        res = PREFIX_0 * 20
        Fuzz.privatekey_to_ETH_address_bytes(dec, res)
        return res

    # ==============================================================================

    def Decimal_To_ETH_Addr_Digest(self, dec):
        """ Privatekey Integer value passed to function. Output is 20 bytes ETH address lowercase without 0x"""
        if dec < 0: dec = MAX_HEX + dec
        pass_int_value = fl(dec).encode('utf8')
        res = self.Decimal_To_ETH_Addr_Bytes(pass_int_value)
        return self.byter(res)

    # =============================================================================

    def Decimal_To_Batch_ETH_Addr(self, dec, m):
        """

        Starting Privatekey Integer value passed to function as pvk_int.
        Integer m is, how many times sequential increment is done from the starting key.
        Output is bytes 20*m of ETH address lowercase without 0x as hex string

        """
        if m <= 0: m = 1
        if dec < 0: dec = MAX_HEX + dec
        start_pvk = fl(dec).encode('utf8')
        res = Fuzz.privatekey_group_to_ETH_address(start_pvk, m)
        addrlist = self.fuzz256k1(res)
        Fuzz.free_memory(res)
        return addrlist

    # ==============================================================================
    def RIPEMD160_To_Addr(self, hash160, addr_type, compress: bool = True):
        """
        Convert RIPEMD160 (Hash160) To Compressed and Uncompressed Address (addr_type: 0=p2pkh, 1=p2sh, 2=bech32)

        :param hash160:
        :param addr_type:
        :type addr_type: int.
        :param compress:
        :type compress: bool.
        :return address:
        :rtype: str.

        ---------------------------------------------------------------------------

        >>> ripemd160 = b'YOUR_HASH160' # example
        >>> compress_address = self.RIPEMD160_To_Addr(ripemd160, 0, True)
        >>> uncompress_address = self.RIPEMD160_To_Addr(ripemd160, 0, False)
        >>> # p2sh valid just compressed key
        >>> compress_p2sh = self.RIPEMD160_To_Addr(ripemd160, 1, True)
        >>> # bech32 valid just compressed key
        >>> compress_bech32 = self.RIPEMD160_To_Addr(ripemd160, 2, True)

        -----------------------------------------------------------------------------


        """
        res = Fuzz.hash_to_address(addr_type, compress, hash160)
        addr = self.fuzz256k1(res)
        Fuzz.free_memory(res)
        return addr

    # ==============================================================================
    def Wif_To_Hex(self, wif):

        """

        Converts a WIF Key to Hex.

        :param wif:
        :type wif: str.
        :return hex:
        :rtype: str.

        =============================================================================

        >>> WIF = "Your_Wif_Key" # example
        >>> hexed = self.Wif_To_Hex(WIF)

        =============================================================================

        """

        pvk = ''
        if wif[0] == '5':
            pvk = self.b58k1.b58_decode(wif)[2:-8]
        elif wif[0] in ['L', 'K']:
            pvk = self.b58k1.b58_decode(wif)[2:-10]
        else:
            raise ValueError("[Error] Incorrect WIF Key")
        return pvk

    # ==============================================================================
    def Wif_To_Decimal(self, wif):
        """

        Converts a WIF Key to Decimal


        :param wif:
        :type wif: str.
        :return deccimal:
        :rtype: int.

        =============================================================================

        >>> WIF = "Your_Wif_Key" # example
        >>> dec = self.Wif_To_Decimal(WIF)

        =============================================================================

        """
        pvk = ''
        pvk_hex = self.Wif_To_Hex(wif)
        if pvk_hex != '': pvk = int(pvk_hex, 16)
        return pvk

    # ==============================================================================
    def Hex_To_Wif(self, pvk: str, compress: bool = True) -> str:
        """
        Input Privatekey can in any 1 of these [Integer] [Hex] [Bytes] form

        :param pvk:
        :type pvk: str.
        :param compress:
        :type compress: bool.
        :return:
        :rtype: str.

        =============================================================================

        >>> pvk = "Your_Privatekey" # example
        >>> hexed = self.Hex_To_Wif(pvk)
        >>> uncompress_wif = self.Hex_To_Wif(pvk, False)
        >>> compress_wif = self.Hex_To_Wif(pvk, True)

        =============================================================================



        """
        inp = ''
        suff = '01' if compress else ''
        if type(pvk) in [int, str]:
            inp = bytes.fromhex('80' + fl(pvk) + suff)
        elif type(pvk) == bytes:
            stuf = bytes.fromhex(suff)
            inp = PREFIX_80 + fl(pvk) + stuf
        else:
            ValueError("[Error] Input Privatekey format [Integer] [Hex] [Bytes] allowed only")
        if inp != '':
            res = self.get_sha256(inp)
            res2 = self.get_sha256(res)
            return self.b58k1.b58_encode(inp + res2[:4])
        else:
            return inp

    # ==============================================================================
    @staticmethod
    def checksum(self, inp):
        """
        Checksum Algorithm Digest Double SHA256
        """
        res = get_sha256(get_sha256(inp))
        return res[:4]

    # ==============================================================================
    def Public_To_Addr(self, pub, addr_type, compress=True):
        res = Fuzz.pubkey_to_address(addr_type, compress, pub)
        addr = self.fuzz256k1(res)
        Fuzz.free_memory(res)
        return addr

    # ==============================================================================

    # ==============================================================================
    @staticmethod
    def Pub_To_RIPEMD160(self, pub: bytes, addr_type: int, compress: bool = True):

        """
        Converts a public key to RIPEMD160.

        public key for p2pkh: addr_type = 0
        ----------------------------------
        public key for p2sh: addr_type = 1
        ----------------------------------
        public key for bech32: addr_type = 2
        ----------------------------------

        :param pub:
        :param addr_type:
        :param compress:
        :return:

        """
        res = PREFIX_0 * 20
        Fuzz.pubkey_to_h160(addr_type, compress, pub, res)
        return res

    # ==============================================================================

    def Pub_To_RIPEMD160_DIGEST(self, pub: bytes, addr_type: int, compress: bool = True):
        res = self.Pub_To_RIPEMD160(pub, addr_type, compress)
        return self.byter(res)

    # ==============================================================================

    def Pub_To_Ethereum_Addr(self, pub: bytes):
        """ 65 Upub bytes input. Output is 20 bytes ETH address lowercase with 0x as hex string"""
        xy = pub[1:]
        res = Fuzz.pubkeyxy_to_ETH_address(xy)
        addr = self.fuzz256k1(res)
        Fuzz.free_memory(res)
        return '0x' + addr

    # ==============================================================================

    @staticmethod
    def Pub_To_Ethereum_Addr_Hash(self, xy):
        """
        65 Upub bytes input. Output is 20 bytes ETH address lowercase with 0x as hex string
        :param xy:
        :return:

        """

        res = PREFIX_0 * 20
        Fuzz.pubkeyxy_to_ETH_address_bytes(xy, res)
        return res

    # ==============================================================================

    def Pub_To_Ethereum_Addr_Digest(self, pub: bytes):
        """ 65 Upub bytes input. Output is 20 bytes ETH address lowercase without 0x"""
        xy = pub[1:]
        res = self.Pub_To_Ethereum_Addr(xy)
        return bytes(bytearray(res))

    # ==============================================================================

    # ==============================================================================

    @staticmethod
    def Load_To_Memory(self, input_bin: str, verbose: bool = False):
        """input_bin_file is sorted h160 data of 20 bytes each element.
        ETH address can also work without 0x if sorted binary format"""
        Fuzz.Load_data_to_memory(input_bin.encode("utf-8"), verbose)

    # ==============================================================================
    @staticmethod
    def Check_Collision(self, RIPEMD160):
        """ h160 is the 20 byte hash to check for collision in data, already loaded in RAM.
        Use the function Load_To_Memory before calling this check"""
        return Fuzz.check_collision(RIPEMD160)

    # ==============================================================================
    @staticmethod
    def Hex_To_Dec(self, hexed: str):
        return int(hexed, 16)

    # =============================================================================

    def Hex_To_Addr(self, hexed: str, compress: bool):
        dec = int(hexed, 16)
        if compress:
            return self.Decimal_To_Addr(dec, addr_type=0, compress=True)
        else:
            return self.Decimal_To_Addr(dec, addr_type=0, compress=False)

    # =============================================================================
    @staticmethod
    def Hex_To_Bytes(self, hexed: str) -> bytes:
        return bytes.fromhex(hexed)

    # =============================================================================

    @staticmethod
    def byter(self, mass):
        return bytes(bytearray(mass))

    # ==============================================================================
    @staticmethod
    def _ScalarMultiply(self, pvk: int):
        """ Integer value passed to function. 65 bytes uncompressed pubkey output """
        res = PREFIX_0 * 65
        pass_int_value = fl(pvk).encode('utf8')
        Fuzz.scalar_multiplication(pass_int_value, res)
        return res

    def ScalarMultiply(self, pvk: int):
        if pvk < 0: pvk = MAX_HEX + pvk
        res = self._ScalarMultiply(pvk)
        return self.byter(res)

    # ==============================================================================
    @staticmethod
    def _ScalarMultiplyList(self, pvklist):
        """
        Integer list passed to function. 65*len bytes uncompressed pubkey output. No Zero Point handling.
        """
        sz = len(pvklist)
        res = PREFIX_0 * (65 * sz)
        pvks = b''.join(pvklist)
        Fuzz.scalar_multiplications(pvks, sz, res)
        return res

    @staticmethod
    def ScalarMultiplyList(self, pvk_list):
        pvk_list = [bytes.fromhex(fl(MAX_HEX + i)) if i < 0 else bytes.fromhex(fl(i)) for i in pvk_list]
        res = self.ScalarMultiplyList(pvk_list)
        return self.byter(res)

    # ==============================================================================
    @staticmethod
    def _PointMultiplication(self, pubkey_bytes, kk):
        """ Input Point and Integer value passed to function. 65 bytes uncompressed pubkey output """
        res = PREFIX_0 * 65
        bytes_value = bytes.fromhex(hex(kk)[2:].zfill(64))  # strict 32 bytes scalar
        Fuzz.point_multiplication(pubkey_bytes, bytes_value, res)
        return res

    def PointMultiplication(self, P, k):
        if type(P) == int: k, P = P, k
        res = self.PointMultiplication(P, k)
        return self.byter(res)

    # ==============================================================================
    @staticmethod
    def _get_X_To_Y(self, x_hex, is_even):
        """ Input x_hex encoded as bytes and bool is_even. 32 bytes y of point output """
        res = PREFIX_0 * 32
        Fuzz.get_x_to_y(x_hex.encode('utf8'), is_even, res)
        return res

    @staticmethod
    def get_X_To_Y(self, x_hex, is_even):
        res = self._get_X_To_Y(x_hex, is_even)
        return self.byter(res)

    # ==============================================================================
    @staticmethod
    def _PointIncrement(self, pubkey_bytes):
        res = PREFIX_0 * 65
        Fuzz.point_increment(pubkey_bytes, res)
        return res

    @staticmethod
    def PointIncrement(self, pubkey_bytes):
        res = self._PointIncrement(pubkey_bytes)
        return self.byter(res)

    # ==============================================================================
    @staticmethod
    def _PointNegation(self, pubkey_bytes):
        res = PREFIX_0 * 65
        Fuzz.point_negation(pubkey_bytes, res)
        return res

    @staticmethod
    def PointNegation(self, pubkey_bytes):
        res = self._PointNegation(pubkey_bytes)
        return self.byter(res)

    # ==============================================================================

    @staticmethod
    def _PointDoubling(self, pubkey_bytes):
        res = PREFIX_0 * 65
        Fuzz.point_doubling(pubkey_bytes, res)
        return res

    def PointDoubling(self, pubkey_bytes):
        res = self._PointDoubling(pubkey_bytes)
        return self.byter(res)

    # ==============================================================================
    @staticmethod
    def init_P2_Group(self, pubkey_bytes):
        Fuzz.init_P2_Group(pubkey_bytes)

    # ==============================================================================



from .lib import Contactor


__name__ = 'SEC256k2'
__author__ = 'Mohammadreza (Mmdrza.com)'
__version__ = '1.3.6'
__license__ = 'MIT'
__copyright__ = 'Copyright Mmdrza.com @ 2023'
__credits__ = ['Mmdrza']
__description__ = 'SECP256k2 Library for generated and Converted Fast Cryptographic Algorithms Python'
__all__ = [
    'Contactor',
    '__name__',
    '__author__',
    '__version__',
    '__license__',
    '__copyright__',
    '__credits__',
    '__description__'
]



import re
from setuptools import setup, find_packages
import os

PACK_NAME = "secp256k2"
PACK_DESCRIPTION = ("Generated and Converted Keys with any Type Foundation from Private Key [WIF Hexed Mnemonic and "
                    "Binary Bytes seed] in Python")
PACK_VERSION = "1.3.6"
PACK_LICENSE = "MIT"
PACK_AUTHOR = "Mohammadreza (Mmdrza.Com)"
PACK_EMAIL = "Pymmdrza@gmail.com"
PACK_URL = "https://github.com/secp256k2/secp256k2"
PACK_ISSUES = 'https://github.com/secp256k2/secp256k2/issues'
PACK_DOCS_URL = 'https://github.com/secp256k2/secp256k2'
PACK_TYPE_README = 'text/markdown'
PACK_KEYWORD = ["secp256k2", "secp256k1", "Wif", "Mnemonic", "Binary", "seed", "Foundation", "Private", "Key", "HEX",
                "Mnemonic", "Binary", "Bytes", "bitcoin", "ethereum", "tron", "dogecoin", "zcash", "digibyte",
                "bitcoin gold", "wallet", "bip32", "bip39", "litecoin", "qtum", "ravencoin", "BTC", "ETH", "TRX",
                "DOGE", "BTG", "LTC", "ZEC", "AXE", "DASH"]
PACK_CLASSIFIERS = ["Development Status :: 5 - Production/Stable",
                    "Intended Audience :: Developers",
                    "Intended Audience :: Information Technology",
                    "Topic :: Security :: Cryptography",
                    "License :: OSI Approved :: MIT License",
                    "Programming Language :: Python :: 3",
                    "Programming Language :: Python :: 3.6",
                    "Programming Language :: Python :: 3.7",
                    "Programming Language :: Python :: 3.8",
                    "Programming Language :: Python :: 3.9",
                    "Programming Language :: Python :: 3.10",
                    "Programming Language :: Python :: 3.11",
                    "Programming Language :: Python :: Implementation :: CPython",
                    "Operating System :: OS Independent"]
PACK_COPYRIGHT = f"Copyright (C) 2023 ~ Mmdrza.Com"

PACK_README = "README.md"
PACK_PROJECT_URLS = {
    "Bug Tracker": PACK_ISSUES,
    "Documentation": PACK_DOCS_URL,
    "Source Code": PACK_URL,
    "Website": "https://mmdrza.com"
}
PACK_SCRIPTS_CONSOLE = {}

data_files = []
if os.name == 'nt':
    data_files.append('secp256k2/_secp256k2.dll')
else:
    data_files.append('secp256k2/_secp256k2.so')


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    with open(os.path.join(package, '__init__.py'), 'rb') as init_py:
        src = init_py.read().decode('utf-8')
        return re.search("__version__ = ['\"]([^'\"]+)['\"]", src).group(1)


version = get_version('secp256k2')

with open(PACK_README, 'r', encoding='utf-8') as readme:
    long_description = readme.read()


setup(
    name=PACK_NAME,
    version=version,
    description=PACK_DESCRIPTION,
    long_description=long_description,
    long_description_content_type=PACK_TYPE_README,
    url=PACK_URL,
    packages=find_packages(),
    include_package_data=True,
    data_files=[('', data_files)],
    project_urls=PACK_PROJECT_URLS,
    classifiers=PACK_CLASSIFIERS,
    entry_points=PACK_SCRIPTS_CONSOLE,
    author=PACK_AUTHOR,
    author_email=PACK_EMAIL,
    license=PACK_LICENSE,
    keywords=PACK_KEYWORD,
    zip_safe=True
)



import ctypes
import math, pickle


argtypes_dict = {
    'scalar_multiplication': [ctypes.c_char_p, ctypes.c_char_p],
    'scalar_multiplications': [ctypes.c_char_p, ctypes.c_int, ctypes.c_char_p],
    'get_x_to_y': [ctypes.c_char_p, ctypes.c_bool, ctypes.c_char_p],
    'point_increment': [ctypes.c_char_p, ctypes.c_char_p],
    'point_negation': [ctypes.c_char_p, ctypes.c_char_p],
    'point_doubling': [ctypes.c_char_p, ctypes.c_char_p],
    'privatekey_to_coinaddress': [ctypes.c_int, ctypes.c_int, ctypes.c_bool, ctypes.c_char_p],
    'privatekey_to_address': [ctypes.c_int, ctypes.c_bool, ctypes.c_char_p],
    'hash_to_address': [ctypes.c_int, ctypes.c_bool, ctypes.c_char_p],
    'pubkey_to_address': [ctypes.c_int, ctypes.c_bool, ctypes.c_char_p],
    'privatekey_to_h160': [ctypes.c_int, ctypes.c_bool, ctypes.c_char_p, ctypes.c_char_p],
    'privatekey_loop_h160': [ctypes.c_ulonglong, ctypes.c_int, ctypes.c_bool, ctypes.c_char_p, ctypes.c_char_p],
    'privatekey_loop_h160_sse': [ctypes.c_ulonglong, ctypes.c_int, ctypes.c_bool, ctypes.c_char_p, ctypes.c_char_p],
    'pubkey_to_h160': [ctypes.c_int, ctypes.c_bool, ctypes.c_char_p, ctypes.c_char_p],
    'pbkdf2_hmac_sha512_dll': [ctypes.c_void_p, ctypes.c_void_p, ctypes.c_int],
    'pbkdf2_hmac_sha512_list': [ctypes.c_void_p, ctypes.c_void_p, ctypes.c_ulonglong, ctypes.c_int, ctypes.c_ulonglong],
    'pub_endo1': [ctypes.c_char_p, ctypes.c_char_p],
    'pub_endo2': [ctypes.c_char_p, ctypes.c_char_p],
    'b58_encode': [ctypes.c_char_p],
    'b58_decode': [ctypes.c_char_p],
    'bech32_address_decode': [ctypes.c_int, ctypes.c_char_p, ctypes.c_char_p],
    'get_sha256': [ctypes.c_char_p],
    'create_baby_table': [ctypes.c_void_p],
    'point_addition': [ctypes.c_char_p, ctypes.c_char_p, ctypes.c_char_p],
    'point_subtraction': [ctypes.c_char_p, ctypes.c_char_p, ctypes.c_char_p],
    'point_loop_subtraction': [ctypes.c_ulonglong, ctypes.c_char_p, ctypes.c_char_p],
    'point_loop_addition': [ctypes.c_ulonglong, ctypes.c_char_p, ctypes.c_char_p],
    'point_vector_addition': [ctypes.c_ulonglong, ctypes.c_char_p, ctypes.c_char_p],
    'point_sequential_increment_P2': [ctypes.c_ulonglong, ctypes.c_char_p, ctypes.c_char_p],
    'point_sequential_increment_P2_mcpu': [ctypes.c_ulonglong, ctypes.c_char_p, ctypes.c_int, ctypes.c_char_p],
    'point_sequential_increment': [ctypes.c_ulonglong, ctypes.c_char_p, ctypes.c_char_p],
    'point_sequential_decrement': [ctypes.c_ulonglong, ctypes.c_char_p, ctypes.c_char_p],
    'pubkeyxy_to_ETH_address': [ctypes.c_char_p],
    'pubkeyxy_to_ETH_address_bytes': [ctypes.c_char_p, ctypes.c_char_p],
    'privatekey_to_ETH_address': [ctypes.c_char_p],
    'privatekey_to_ETH_address_bytes': [ctypes.c_char_p, ctypes.c_char_p],
    'privatekey_group_to_ETH_address': [ctypes.c_char_p],
    'privatekey_group_to_ETH_address_bytes': [ctypes.c_char_p, ctypes.c_char_p],
    'init_P2_Group': [ctypes.c_char_p],
    'free_memory': [ctypes.c_void_p],
    'bloom_check_add': [ctypes.c_void_p, ctypes.c_int, ctypes.c_int, ctypes.c_ulonglong, ctypes.c_ubyte, ctypes.c_char_p],
    'bloom_batch_add': [ctypes.c_void_p, ctypes.c_int, ctypes.c_int, ctypes.c_ulonglong, ctypes.c_ubyte, ctypes.c_char_p],
    'bloom_check_add_mcpu': [ctypes.c_void_p, ctypes.c_int, ctypes.c_int, ctypes.c_ulonglong, ctypes.c_ubyte, ctypes.c_char_p],
    'create_bsgs_bloom_mcpu': [ctypes.c_int, ctypes.c_ulonglong, ctypes.c_ulonglong, ctypes.c_ubyte, ctypes.c_char_p],
    'bsgs_2nd_check_prepare': [ctypes.c_ulonglong],
    'bsgs_2nd_check': [ctypes.c_char_p, ctypes.c_char_p, ctypes.c_ulonglong, ctypes.c_char_p],
    'Load_data_to_memory': [ctypes.c_char_p, ctypes.c_bool],
    'check_collision': [ctypes.c_char_p]

}

restype_dict = {
    'privatekey_to_coinaddress': ctypes.c_void_p,
    'privatekey_to_address': ctypes.c_void_p,
    'hash_to_address': ctypes.c_void_p,
    'pubkey_to_address': ctypes.c_void_p,
    'pubkey_to_h160': ctypes.c_void_p,
    'pub_endo1': ctypes.c_void_p,
    'pub_endo2': ctypes.c_void_p,
    'b58_encode': ctypes.c_void_p,
    'b58_decode': ctypes.c_void_p,
    'bech32_address_decode': ctypes.c_void_p,
    'get_sha256': ctypes.c_void_p,
    'create_baby_table': ctypes.c_void_p,
    'point_addition': ctypes.c_void_p,
    'point_subtraction': ctypes.c_void_p,
    'point_loop_subtraction': ctypes.c_void_p,
    'point_loop_addition': ctypes.c_void_p,
    'point_vector_addition': ctypes.c_void_p,
    'point_sequential_increment_P2': ctypes.c_void_p,
    'point_sequential_increment_P2_mcpu': ctypes.c_void_p,
    'point_sequential_increment': ctypes.c_void_p,
    'point_sequential_decrement': ctypes.c_void_p,
    'pubkeyxy_to_ETH_address': ctypes.c_void_p,
    'pubkeyxy_to_ETH_address_bytes': ctypes.c_void_p,
    'privatekey_to_ETH_address': ctypes.c_void_p,
    'privatekey_to_ETH_address_bytes': ctypes.c_void_p,
    'privatekey_group_to_ETH_address': ctypes.c_void_p,
    'bloom_check_add': ctypes.c_void_p,
    'bsgs_2nd_check': ctypes.c_void_p,
    'check_collision': ctypes.c_bool

}


